import os.path
import shutil


def isdir(path: str) -> bool:
    return os.path.isdir(path)


def isfile(path: str) -> bool:
    return os.path.isfile(path)


def move(src, dest):
    shutil.move(src, dest)


def delete_file(path: str):
    if isfile(path):
        os.remove(path)



WRITESONIC_BASE_URL = 'https://api.writesonic.com'
CHATSONIC_URL_PATH = '/v2/business/content/chatsonic'
WRITESONIC_ENGINE_PREMIUM = 'premium'
WRITESONIC_ENGINE_SUPERIUM = 'superium'
WRITESONIC_ENGINES = (WRITESONIC_ENGINE_PREMIUM, WRITESONIC_ENGINE_SUPERIUM)
WRITESONIC_LANGUAGE_ENGLISH = 'en'
WRITESONIC_LANGUAGE_CHINESE = 'zh'
WRITESONIC_LANGUAGES = (WRITESONIC_LANGUAGE_ENGLISH, WRITESONIC_LANGUAGE_CHINESE)



import os
from dataclasses import dataclass, field
from typing import List

import requests
from edi_core.ai.writesonic import *


@dataclass(kw_only=True)
class ChatSonicConfig:
    api_key: str = os.environ.get("SONIC_KEY")
    base_url: str = WRITESONIC_BASE_URL
    path: str = CHATSONIC_URL_PATH
    engine: str = WRITESONIC_ENGINE_PREMIUM
    language: str = WRITESONIC_LANGUAGE_ENGLISH
    enable_memory: bool = False
    enable_google_results: bool = False

    def __post_init__(self):
        if not self.api_key:
            raise ValueError("token is required")
        if self.engine not in WRITESONIC_ENGINES:
            raise ValueError("engine is invalid")
        if self.language not in WRITESONIC_LANGUAGES:
            raise ValueError("language is invalid")

    def get_request_url(self):
        return f'{self.base_url}{self.path}'


@dataclass
class ChatSonicPayload:
    @dataclass
    class Message:
        is_sent: bool
        message: str

    input_text: str
    history_data: List[Message] = field(default_factory=list)
    enable_memory: bool = False
    enable_google_results: bool = False

    def push_response(self, message: str):
        message_sent = self.Message(is_sent=True, message=self.input_text)
        message_received = self.Message(is_sent=False, message=message)
        self.history_data.append(message_sent)
        self.history_data.append(message_received)
        self.input_text = ''

    def set_config(self, config: ChatSonicConfig):
        self.enable_memory = config.enable_memory
        self.enable_google_results = config.enable_google_results


@dataclass
class ChatSonicResponse:
    message: str
    image_urls: List[str] = field(default_factory=list)

    @staticmethod
    def from_json(json_data):
        return ChatSonicResponse(
            message=json_data['message'],
            image_urls=json_data['image_urls']
        )


class ChatSonic:
    _config: ChatSonicConfig
    _session = requests.Session()

    def __init__(self, config: ChatSonicConfig):
        self._config = config

    def call(self, payload: ChatSonicPayload) -> (ChatSonicResponse, ChatSonicPayload):
        url = self._config.get_request_url()

        raw_response = self._session.post(
            url=url,
            headers=self.get_request_headers(),
            params=self.get_request_params(),
            json=payload.__dict__,
            timeout=60
        )

        raw_response.raise_for_status()

        response = ChatSonicResponse.from_json(raw_response.json())
        payload.push_response(response.message)

        return response, payload

    def chat(self, message: str) -> (ChatSonicResponse, ChatSonicPayload):
        payload = ChatSonicPayload(input_text=message);
        payload.set_config(self._config)
        return self.call(payload)

    def get_request_headers(self):
        return {'X-API-KEY': self._config.api_key}

    def get_request_params(self):
        return {
            'engine': self._config.engine,
            'language': self._config.language
        }



from typing import List

from tinydb import TinyDB
from tinydb.table import Document


class Tiny:
    _db = None

    def __init__(self, db_file):
        self._db = TinyDB(db_file)

    def insert(self, data: dict) -> int:
        return self._db.insert(data)

    def all(self) -> List[Document]:
        return self._db.all()

    def search(self, query) -> List[Document]:
        return self._db.search(query)

    def close(self):
        self._db.close()



from typing import Dict, Any

from edi_core.parser.toml_parser import parse_toml_file


class PyProject:
    _root: str
    _config: Dict[str, Any]
    _config_file_name = "pyproject.toml"

    def __init__(self, path: str):
        self.path = path
        self._config = parse_toml_file(f'{path}/{self._config_file_name}')



import hashlib

algorithms_available = hashlib.algorithms_available


def hash_str(data: str, algo='md5') -> str:
    h = hashlib.new(algo)
    h.update(data.encode('utf-8'))
    return h.hexdigest()



import toml


def parse_toml_file(toml_file_path: str) -> dict:
    with open(toml_file_path, 'r') as file:
        return toml.load(file)


def parse_toml_str(toml_str: str) -> dict:
    return toml.loads(toml_str)


def save_to_toml_file(path: str, data: dict):
    with open(path, 'w') as file:
        toml.dump(data, file)



directions = [(0, 1), (1, 0), (0, -1), (-1, 0)]


class SliderPuzzle:

    def solve(self):
        return


class SearchNode:
    board: list
    free: tuple

    step: 0

    def get_distance(self):
        distance = 0
        for i in range(len(self.board)):
            for j in range(len(self.board[i])):
                if self.board[i][j] != i * len(self.board[i]) + j + 1:
                    distance += abs(i - (self.board[i][j] - 1) // len(self.board[i])) + abs(
                        j - (self.board[i][j] - 1) % len(self.board[i]))
        return distance + self.step


